@@ -1,16 +1,17 @@
 package processing
 
 // DCSO FEVER
-// Copyright (c) 2017, 2019, DCSO GmbH
+// Copyright (c) 2017, 2020, DCSO GmbH
 
 import (
-	"encoding/json"
+	"crypto/tls"
 	"net"
 	"sync"
 	"time"
 
 	"github.com/DCSO/fever/types"
 	"github.com/DCSO/fever/util"
+	"github.com/buger/jsonparser"
 
 	log "github.com/sirupsen/logrus"
 )
@@ -28,8 +29,11 @@ type ForwardHandler struct {
 	Logger              *log.Entry
 	DoRDNS              bool
 	RDNSHandler         *RDNSHandler
+	AddedFields         map[string]string
 	ContextCollector    *ContextCollector
+	StenosisConnector   *StenosisConnector
 	ForwardEventChan    chan []byte
+	FlowNotifyChan      chan types.Entry
 	OutputSocket        string
 	OutputConn          net.Conn
 	Reconnecting        bool
@@ -177,35 +181,46 @@ func MakeForwardHandler(reconnectTimes int, outputSocket string) *ForwardHandler
 	return fh
 }
 
-// Consume processes an Entry and forwards it
+// Consume processes an Entry and prepares it to be sent off to the
+// forwarding sink
 func (fh *ForwardHandler) Consume(e *types.Entry) error {
 	doForwardThis := util.ForwardAllEvents || util.AllowType(e.EventType)
 	if doForwardThis {
-		var ev types.EveOutEvent
-		err := json.Unmarshal([]byte(e.JSONLine), &ev)
-		if err != nil {
-			return err
-		}
+		// mark flow as relevant when alert is seen
 		if GlobalContextCollector != nil && e.EventType == types.EventTypeAlert {
 			GlobalContextCollector.Mark(string(e.FlowID))
 		}
+		// we also perform active rDNS enrichment if requested
 		if fh.DoRDNS && fh.RDNSHandler != nil {
-			err = fh.RDNSHandler.Consume(e)
+			err := fh.RDNSHandler.Consume(e)
 			if err != nil {
 				return err
 			}
-			ev.SrcHost = e.SrcHosts
-			ev.DestHost = e.DestHosts
 		}
-		var jsonCopy []byte
-		jsonCopy, err = json.Marshal(ev)
-		if err != nil {
-			return err
+		for k, v := range fh.AddedFields {
+			val, err := util.EscapeJSON(v)
+			if err != nil {
+				fh.Logger.Warningf("cannot escape value: %s", v)
+				continue
+			}
+			newJSON, err := jsonparser.Set([]byte(e.JSONLine), val, k)
+			if err != nil {
+				fh.Logger.Warningf("cannot set %s: %s", k, v)
+				continue
+			} else {
+				e.JSONLine = string(newJSON)
+			}
+		}
+		// if we use Stenosis, the Stenosis connector will take ownership of
+		// alerts
+		if fh.StenosisConnector != nil && e.EventType == types.EventTypeAlert {
+			fh.StenosisConnector.Accept(e)
+		} else {
+			fh.ForwardEventChan <- []byte(e.JSONLine)
+			fh.Lock.Lock()
+			fh.PerfStats.ForwardedPerSec++
+			fh.Lock.Unlock()
 		}
-		fh.ForwardEventChan <- jsonCopy
-		fh.Lock.Lock()
-		fh.PerfStats.ForwardedPerSec++
-		fh.Lock.Unlock()
 	}
 	return nil
 }
@@ -228,7 +243,21 @@ func (fh *ForwardHandler) GetEventTypes() []string {
 // IPs in outgoing EVE events.
 func (fh *ForwardHandler) EnableRDNS(expiryPeriod time.Duration) {
 	fh.DoRDNS = true
-	fh.RDNSHandler = MakeRDNSHandler(util.NewHostNamer(expiryPeriod, 2*expiryPeriod))
+	fh.RDNSHandler = MakeRDNSHandler(util.NewHostNamerRDNS(expiryPeriod, 2*expiryPeriod))
+}
+
+// AddFields enables the addition of a custom set of top-level fields to the
+// forwarded JSON.
+func (fh *ForwardHandler) AddFields(fields map[string]string) {
+	fh.AddedFields = fields
+}
+
+// EnableStenosis ...
+func (fh *ForwardHandler) EnableStenosis(endpoint string, timeout, timeBracket time.Duration,
+	notifyChan chan types.Entry, cacheExpiry time.Duration, tlsConfig *tls.Config) (err error) {
+	fh.StenosisConnector, err = MakeStenosisConnector(endpoint, timeout, timeBracket,
+		notifyChan, fh.ForwardEventChan, cacheExpiry, tlsConfig)
+	return
 }
 
 // Run starts forwarding of JSON representations of all consumed events
