@@ -1,9 +1,12 @@
 package util
 
 // DCSO FEVER
-// Copyright (c) 2017, 2018, DCSO GmbH
+// Copyright (c) 2017, 2018, 2020, DCSO GmbH
 
 import (
+	"crypto/tls"
+	"crypto/x509"
+	"encoding/json"
 	"io/ioutil"
 	"math/rand"
 	"os"
@@ -47,6 +50,15 @@ var evekeys = [][]string{
 	[]string{"flow_id"},                // 22
 }
 
+// EscapeJSON escapes a string as a quoted byte slice for direct use in jsonparser.Set().
+func EscapeJSON(i string) ([]byte, error) {
+	b, err := json.Marshal(i)
+	if err != nil {
+		return []byte(""), err
+	}
+	return b, nil
+}
+
 // ParseJSON extracts relevant fields from an EVE JSON entry into an Entry struct.
 func ParseJSON(json []byte) (e types.Entry, parseerr error) {
 	e = types.Entry{}
@@ -263,3 +275,37 @@ func RandStringBytesMaskImprSrc(n int) string {
 
 	return string(b)
 }
+
+// MakeTLSConfig returns a TLS configuration suitable for an endpoint with private
+// key stored in keyFile and corresponding certificate stored in certFile. rcas
+// defines a list of root CA filenames.
+// If certFile and keyFile are empty, e.g., when configuring a tls-client
+// endpoint w/o mutual authentication, only the RootCA pool is populated.
+// Note: It appears as if ICAs have to be loaded via a chained server
+// certificate file as the RootCAs pool in tls.Config appears to be referred to
+// for RCAs only.
+func MakeTLSConfig(certFile, keyFile string, rcas []string, skipVerify bool) (*tls.Config, error) {
+	certs := make([]tls.Certificate, 0, 1)
+
+	if certFile != "" && keyFile != "" {
+		c, err := tls.LoadX509KeyPair(certFile, keyFile)
+		if err != nil {
+			return nil, err
+		}
+		certs = append(certs, c)
+	}
+	rcaPool := x509.NewCertPool()
+	for _, filename := range rcas {
+		rca, err := ioutil.ReadFile(filename)
+		if err != nil {
+			return nil, err
+		}
+		rcaPool.AppendCertsFromPEM(rca)
+	}
+
+	return &tls.Config{
+		Certificates:       certs,
+		RootCAs:            rcaPool,
+		InsecureSkipVerify: skipVerify,
+	}, nil
+}
